/**
 * MCS Media Computer Software
 * Copyright 2018 by Wilfried Klaas
 * Project: SPSEmulator
 * File: Not.java
 * EMail: W.Klaas@gmx.de
 * Created: 25.11.2018 wklaa_000
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package de.mcs.tools.sps.mnemonic;

import java.util.EnumSet;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

//@formatter:off
/**
 * STA PORT: output register A to digital out, all bits
 * STA DOUTx: output register A lowest bit to digital out x
 * STA SRVx: output register A to SRV #x. x should be 1 or 2
 * STA PWMx: output register A to PWM #x. x should be 1 or 2
 * @author wklaa_000
 *
 */
//@formatter:on
public class STA extends AbstractMnemonic implements Mnemonic {

  private static final int PWM = 0x59;
  private static final int SRV = 0x5B;
  byte value;
  static int DOUT = 0x54;

  enum OUTPUTSOURCE {
    DIGITAL, PWM, SRV
  };

  OUTPUTSOURCE inputSource;

  public STA(String line) throws SyntaxError {
    super(line);
    value = 0;
  }

  @Override
  public void checkArgument() throws SyntaxError {
    if (StringUtils.isEmpty(getArgument())) {
      throw new SyntaxError(getLineNumber(),
          String.format("missing argument for %s.", this.getClass().getSimpleName()));
    }
    // int myValue = getArgumentAsNumber();
    // if ((myValue < 1) || (myValue > 2)) {
    // throw new IllegalArgument(getLineNumber(),
    // String.format("argument %s is not in range 1..2 for %s.", getArgument(), this.getClass().getSimpleName()));
    // }
    // value = (byte) ((myValue - 1) & 0xFF);
    String inputString = getArgument().toUpperCase();
    if (inputString.startsWith("DOUT")) {
      inputSource = OUTPUTSOURCE.DIGITAL;
      if ("DOUT".equalsIgnoreCase(inputString)) {
        value = 0;
      } else {
        int myValue = Integer.parseInt(inputString.substring(4));
        if ((myValue < 1) || (myValue > 4)) {
          throw new IllegalArgument(getLineNumber(), String.format("argument %s is not in range DOUT1..4 for %s.",
              getArgument(), this.getClass().getSimpleName()));
        }
        value = (byte) (myValue & 0xFF);
      }
    } else if (inputString.startsWith("PWM")) {
      inputSource = OUTPUTSOURCE.PWM;
      int myValue = 0;
      try {
        myValue = Integer.parseInt(inputString.substring(3));
      } catch (NumberFormatException e) {
      }
      if ((myValue < 1) || (myValue > 2)) {
        throw new IllegalArgument(getLineNumber(), String.format("argument %s is not in range PWM1..2 for %s.",
            getArgument(), this.getClass().getSimpleName()));
      }
      value = (byte) ((myValue - 1) & 0xFF);
    } else if (inputString.startsWith("SRV")) {
      inputSource = OUTPUTSOURCE.SRV;
      int myValue = 0;
      try {
        myValue = Integer.parseInt(inputString.substring(3));
      } catch (NumberFormatException e) {
      }
      if ((myValue < 1) || (myValue > 2)) {
        throw new IllegalArgument(getLineNumber(), String.format("argument %s is not in range SRV1..2 for %s.",
            getArgument(), this.getClass().getSimpleName()));
      }
      value = (byte) ((myValue - 1) & 0xFF);
    } else {
      throw new IllegalArgument(getLineNumber(),
          String.format("illegal argument %s in %s", getArgument(), this.getClass().getSimpleName()));
    }
  }

  @Override
  public int getByte() {
    switch (inputSource) {
    case DIGITAL:
      return DOUT + value;
    case PWM:
      return PWM + value;
    case SRV:
      return SRV + value;
    }
    return 0x00;
  }

  @Override
  public boolean isUsingPage() {
    return false;
  }

  @Override
  public boolean hasArgument() {
    return true;
  }

  @Override
  public Set<HARDWARE> allowedHardware() {
    switch (inputSource) {
    case DIGITAL:
    case PWM:
      return super.allowedHardware();
    case SRV:
      return EnumSet.of(HARDWARE.ARDUINOSPS, HARDWARE.TINYSPS);
    default:
      return super.allowedHardware();
    }
  }

}
